/*
 * test
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <stdio.h>
#include "test.h"

#define METHOD_BATTERY_TIMETOFULL       "TimeToFull"
#define METHOD_BATTERY_TIMETOEMPTY      "TimeToEmpty"
#define METHOD_BATTERY_SETLOWBATLEVEL   "SetLowbatLevel"
#define METHOD_BATTERY_GETLOWBATLEVEL   "GetLowbatLevel"
#define METHOD_BATTERY_CHARGERSTATUS    "ChargerStatus"
#define METHOD_BATTERY_CHARGENOW        "ChargeNow"
#define METHOD_BATTERY_STATUSLOW        "BatteryStatusLow"
#define METHOD_BATTERY_GETPERCENT       "GetPercent"
#define METHOD_BATTERY_GETPERCENTRAW    "GetPercentRaw"
#define METHOD_BATTERY_ISFULL           "IsFull"
#define METHOD_BATTERY_HEALTH           "GetHealth"
#define METHOD_BATTERY_GETINFO          "GetBatteryInfo"
#define METHOD_BATTERY_POWERSUPPLY      "power_supply"

#define CHARGEFULL_NAME     "Full"
#define CHARGENOW_NAME      "Charging"
#define DISCHARGE_NAME      "Discharging"
#define NOTCHARGE_NAME      "Not charging"

#define GOOD_NAME           "Good"
#define OVERHEAT_NAME       "Overheat"
#define TEMPCOLD_NAME       "Cold"
#define OVERVOLT_NAME       "Over voltage"

#define POWER_SUPPLY_TYPE_BATTERY   "1"
#define POWER_SUPPLY_TYPE_UPS       "2"

#define PRESENT_ABNORMAL    "0"
#define PRESENT_NORMAL      "1"

#define MISC_ABNORMAL       "1"
#define MISC_NORMAL         "0"

#define FREQ_STRENGTH_LEVEL_0 "0"
#define FREQ_STRENGTH_LEVEL_1 "50"
#define FREQ_STRENGTH_LEVEL_2 "100"

#define DEFAULT_LOWBATLEVEL 15

#define S_ENTER	1
#define S_LEAVE	0

static struct power_supply_type {
	char *scenario;
	int status;
	char *capacity;
	char *charge_status;
	char *health;
	char *online;
	char *present;
	char *misc;
	char *freq_strength;
	char *name;
} power_supply_types[] = {
	{"norm", S_ENTER, "100",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "CHARGE"},
	{"norm", S_ENTER, "100",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},
	{"norm", S_LEAVE, "100",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"heat1", S_ENTER, "100",   DISCHARGE_NAME, OVERHEAT_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL},
	{"heat1", S_ENTER, "100",   NOTCHARGE_NAME, OVERHEAT_NAME,  POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "HEALTH(H) BEFORE CHARGE"},
	{"heat1", S_LEAVE, "100",   DISCHARGE_NAME, OVERHEAT_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},
	{"heat1", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"heat2", S_ENTER, "100",   CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL},
	{"heat2", S_ENTER, "100",   NOTCHARGE_NAME, OVERHEAT_NAME,  POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "HEALTH(H) AFTER CHARGE"},
	{"heat2", S_LEAVE, "100",   DISCHARGE_NAME, OVERHEAT_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},
	{"heat2", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"cold1", S_ENTER, "100",   DISCHARGE_NAME, TEMPCOLD_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL},
	{"cold1", S_ENTER, "100",   NOTCHARGE_NAME, TEMPCOLD_NAME,  POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "HEALTH(L) BEFORE CHARGE"},
	{"cold1", S_LEAVE, "100",   DISCHARGE_NAME, TEMPCOLD_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},
	{"cold1", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"cold2", S_ENTER, "100",   CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL},
	{"cold2", S_ENTER, "100",   NOTCHARGE_NAME, TEMPCOLD_NAME,  POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "HEALTH(L) AFTER CHARGE"},
	{"cold2", S_LEAVE, "100",   DISCHARGE_NAME, TEMPCOLD_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},
	{"cold2", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"ovp", S_ENTER, "100",    DISCHARGE_NAME, OVERVOLT_NAME,  POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "OVP"},
	{"ovp", S_LEAVE, "100",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"pres1", S_ENTER, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_ABNORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL},
	{"pres1", S_ENTER, "100",   NOTCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_ABNORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "PRESENT BEFORE CHARGE"},
	{"pres1", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_ABNORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},
	{"pres1", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL,   MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"pres2", S_ENTER, "100",   CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL},
	{"pres2", S_ENTER, "100",   NOTCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_ABNORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0,  "PRESENT AFTER CHARGE"},
	{"pres2", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_ABNORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0,  "DISCHARGE"},
	{"pres2", S_LEAVE, "100",   DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, NULL}, /* init */

	{"bat15", S_ENTER, "15",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 15%"}, /* lowbat 15% */
	{"bat15", S_LEAVE, "15",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 15%"},
	{"bat5", S_ENTER, "5",	    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 5%"},  /* lowbat 5% */
	{"bat5", S_LEAVE, "5",	    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 5%"},
	{"bat3", S_ENTER, "3",	    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 3%"},  /* lowbat 3% */
	{"bat3", S_LEAVE, "3",	    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 3%"},
	{"bat1", S_ENTER, "1",	    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 1%"},  /* lowbat 1% */
	{"bat1", S_LEAVE, "1",	    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "LOWBAT 1%"},

	{"ta",   S_ENTER, "100",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "CHARGE"},   /* charging */
	{"ta",  S_LEAVE, "100",		DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},/* discharging */

	{"full", S_ENTER, "100",    CHARGEFULL_NAME, GOOD_NAME,     POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "CHARGE"},   /* full */
	{"full", S_LEAVE, "100",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"},/* discharging */

	{"capa", S_ENTER, "100",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "CAPACITY"},/* discharging */
	{"capa", S_LEAVE, "100",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "CAPACITY"},/* charging */

	{"freq", S_ENTER, "100",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "CHARGE"},
	{"freq", S_ENTER, "100",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_1, "FREQ LEVEL 1 AFTER CHARGE"},
	{"freq", S_ENTER, "100",    CHARGENOW_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_UPS,      PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_2, "FREQ LEVEL 2 AFTER CHARGE"},
	{"freq", S_LEAVE, "100",    DISCHARGE_NAME, GOOD_NAME,      POWER_SUPPLY_TYPE_BATTERY,  PRESENT_NORMAL, MISC_NORMAL, FREQ_STRENGTH_LEVEL_0, "DISCHARGE"}, /* init */
};

static bool get_battery_method(const char *method, int *value)
{
	GVariant *msg = NULL;
	int val = 0;
	char* health = NULL;
	bool ret = FALSE;

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
			DEVICED_PATH_BATTERY,
			DEVICED_INTERFACE_BATTERY,
			method, NULL);
	if (!msg) {
		_E("Failed to call dbus method(%s): No reply.", method);
		return ret;
	}

	if (!strncmp(method, METHOD_BATTERY_HEALTH, strlen(METHOD_BATTERY_HEALTH))) {
		if (!dh_get_param_from_var(msg, "(s)", &health)) {
			_E("Failed to call dbus method(%s): No message.", METHOD_BATTERY_HEALTH);
		} else {
			_I("Success. %s=%s", METHOD_BATTERY_HEALTH, health);
			ret = TRUE;
		}
	}
	else
	{
		if (!dh_get_param_from_var(msg, "(i)", &val)) {
			_E("Failed to call dbus method(%s): No message.", method);
		} else {
			_I("Success. %s=%d", method, val);
			ret = TRUE;
			if (NULL != value)
				*value = val;
		}
	}

	if (health)
		g_free(health);
	g_variant_unref(msg);

	return ret;
}

static bool get_battery_method_vconf(const char *method)
{
	GVariant *msg = NULL;
	int val;
	bool ret = FALSE;

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
			DEVICED_PATH_BATTERY,
			DEVICED_INTERFACE_BATTERY,
			method, NULL);
	if (!msg) {
		_E("Failed to call dbus method(%s): No reply.", method);
		return ret;
	}

	if (!dh_get_param_from_var(msg, "(i)", &val))
		_E("Failed to call dbus method(%s): No message.", method);
	else {
		if (val == -EIO) {
			_E("Failed to call dbus method(%s): Return fail.", method);
		} else {
			_I("Success. %s=%d", method, val);
			ret = TRUE;
		}
	}

	g_variant_unref(msg);
	return ret;
}

static bool get_battery_time_to_full(void)
{
	return get_battery_method(METHOD_BATTERY_TIMETOFULL, NULL);
}

static bool get_battery_time_to_empty(void)
{
	return get_battery_method(METHOD_BATTERY_TIMETOEMPTY, NULL);
}

static bool get_battery_low_level(int *value)
{
	return get_battery_method(METHOD_BATTERY_GETLOWBATLEVEL, value);
}

static bool set_battery_low_level(int newlevel)
{
	GVariant *msg;
	int getlevel = -1;
	int val = -1;
	bool ret = FALSE;

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
			DEVICED_PATH_BATTERY,
			DEVICED_INTERFACE_BATTERY,
			METHOD_BATTERY_SETLOWBATLEVEL, g_variant_new("(i)", newlevel));

	if (!msg) {
		_E("Failed to call dbus method(%s): No reply", METHOD_BATTERY_SETLOWBATLEVEL);
		return ret;
	}

	if (!dh_get_param_from_var(msg, "(i)", &val)) {
		_E("Failed to call dbus method(%s): No message", METHOD_BATTERY_SETLOWBATLEVEL);
		goto out;
	} else {
		if (newlevel <= DEFAULT_LOWBATLEVEL) {
			if (0 != val) {
				_I("Success. Can't set low level(%d) smaller than default(%d).", newlevel, DEFAULT_LOWBATLEVEL);
				ret = TRUE;
			} else {
				_E("Failed. Can't set low level(%d) smaller than default(%d), but returned success.", newlevel, DEFAULT_LOWBATLEVEL);
			}
			goto out;
		} else {
			if (0 != val) {
				_E("Failed to call dbus method(%s): Return fail.", METHOD_BATTERY_SETLOWBATLEVEL);
				goto out;
			}
		}
	}

	if (!get_battery_low_level(&getlevel))   {
		_E("Failed to get battery low level.");
	} else {
		if (getlevel == newlevel) {
			_I("Success : battery low level=%d", newlevel);
			ret = TRUE;
		} else {
			_E("Failed to set low level(%d). Current low level is %d.", newlevel, getlevel);
		}
	}
out:
	g_variant_unref(msg);
	return ret;
}

static bool get_battery_charger_status()
{
	return get_battery_method_vconf(METHOD_BATTERY_CHARGERSTATUS);
}

static bool get_battery_charge_now()
{
	return get_battery_method(METHOD_BATTERY_CHARGENOW, NULL);
}

static bool get_battery_status_low()
{
	return get_battery_method_vconf(METHOD_BATTERY_STATUSLOW);
}

static bool get_battery_percent()
{
	return get_battery_method(METHOD_BATTERY_GETPERCENT, NULL);
}

static bool get_battery_percent_raw()
{
	return get_battery_method(METHOD_BATTERY_GETPERCENTRAW, NULL);
}

static bool get_battery_is_full()
{
	return get_battery_method(METHOD_BATTERY_ISFULL, NULL);
}

static bool get_battery_health()
{
	return get_battery_method(METHOD_BATTERY_HEALTH, NULL);
}

static bool get_battery_info()
{
	GVariant *msg;
	int val[9];
	bool ret = FALSE;
	char *argv[3];

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
			DEVICED_PATH_BATTERY,
			DEVICED_INTERFACE_BATTERY,
			METHOD_BATTERY_GETINFO, NULL);
	if (!msg) {
		_E("Failed to call dbus method(%s): No reply", METHOD_BATTERY_GETINFO);
		return ret;
	}

	if (!dh_get_param_from_var(msg, "(isssiiiiiiii)", &val[0],  //return value
							&argv[0],//status
							&argv[1],//health
							&argv[2],//power source
							&val[1],  //online
							&val[2],  //present
							&val[3],  //capacity
							&val[4],  //current_now
							&val[5], //current_average
							&val[6], //voltage_now
							&val[7], //voltage_average
							&val[8])) //temperature
		_E("Failed to call dbus method(%s): No message", METHOD_BATTERY_GETINFO);
	else {
		if (val[0] < 0) {
			_E("Failed to call dbus method(%s): Return fail", METHOD_BATTERY_GETINFO);
		} else {
			_I("Success: %s %s %s %d %d %d %d %d %d %d %d",
					argv[0], argv[1], argv[2], val[1], val[2],
					val[3], val[4], val[5], val[6], val[7],
					val[8]);
			ret = TRUE;
		}
		g_free(argv[0]);
		g_free(argv[1]);
		g_free(argv[2]);
	}

	g_variant_unref(msg);

	return ret;
}

static bool set_battery_power_supply(int index)
{
	GVariant *msg;
	int val;
	bool ret = FALSE;
	GVariant *var;

	var = g_variant_new("(sisssssss)", POWER_SUBSYSTEM,
				7,
				power_supply_types[index].capacity,
				power_supply_types[index].charge_status,
				power_supply_types[index].health,
				power_supply_types[index].online,
				power_supply_types[index].present,
				power_supply_types[index].misc,
				power_supply_types[index].freq_strength);
				_I("C(%s , %s) P(%s) STATUS(%s) HEALTH(%s)",
				power_supply_types[index].capacity,
				power_supply_types[index].online,
				power_supply_types[index].present,
				power_supply_types[index].charge_status,
				power_supply_types[index].health);

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
						DEVICED_PATH_BATTERY,
						DEVICED_INTERFACE_BATTERY,
						POWER_SUBSYSTEM, var);
	if (!msg) {
		_E("Failed to call dbus method(%s): No reply", POWER_SUBSYSTEM);
		return ret;
	}

	if (!dh_get_param_from_var(msg, "(i)", &val)) {
		_E("Failed to call dbus method(%s): No message", POWER_SUBSYSTEM);
		val = -EBADMSG;
	} else {
		if (val < 0) {
			_E("Failed to call dbus method(%s): Return fail", POWER_SUBSYSTEM);
		} else {
			_I("Success. val=%d", val);
			ret = TRUE;
		}
	}

	/*
	if (power_supply_types[index].name != NULL)
		_I("++++++++++[START] %s ++++++++++", power_supply_types[index].name);
		_I("C(%s , %s) P(%s) STATUS(%s) HEALTH(%s)",
			power_supply_types[index].capacity,
			power_supply_types[index].online,
			power_supply_types[index].present,
			power_supply_types[index].charge_status,
			power_supply_types[index].health);
	if (power_supply_types[index].name != NULL)
		_I("++++++++++[END] %s ++++++++++", power_supply_types[index].name);
	if (val < 0)
		_R("[NG] ---- %s", __func__);
	else
		_R("[OK] ---- %s     : C(%s , %s) P(%s) S(%s) H(%s)",
				__func__,
				power_supply_types[index].capacity,
				power_supply_types[index].online,
				power_supply_types[index].present,
				power_supply_types[index].charge_status,
				power_supply_types[index].health);
	*/
	g_variant_unref(msg);
	sleep(TEST_WAIT_TIME_INTERVAL);

	return ret;
}

static void full_scenario(char *unit, char *capacity, int status)
{
	int index;
	int len;

	if (!unit || !capacity)
		return;

	len = strlen(unit);

	for (index = 0; index < ARRAY_SIZE(power_supply_types); index++) {
		if (len != strlen(power_supply_types[index].scenario))
			continue;
		if (strncmp(unit, power_supply_types[index].scenario, len) != 0 ||
		    power_supply_types[index].status != status)
			continue;
		power_supply_types[index].capacity = capacity;
		_D("%s", power_supply_types[index].capacity);
		set_battery_power_supply(index);
	}
}

void battery_scenario(char *unit, int status)
{
	int index;
	int found = 0;
	int unit_len;
	char *scenario = NULL;

	if (!unit)
		return;

	unit_len = strlen(unit);

	for (index = 0; index < ARRAY_SIZE(power_supply_types); index++) {
		if (unit_len != strlen(power_supply_types[index].scenario))
			continue;
		scenario = power_supply_types[index].scenario;
		if (strncmp(unit, scenario, unit_len) != 0 ||
		    power_supply_types[index].status != status)
			continue;
		found = 1;
		set_battery_power_supply(index);
	}

	if (found)
		return;

	index = strlen(unit);
	if (index < 0 || index > 3)
		return;

	index = strtol(unit, NULL, 10);
	if (index < 0 || index > 100)
		return;

	for (index = 0; index < ARRAY_SIZE(power_supply_types); index++){
		if (strncmp("capa", power_supply_types[index].scenario, 4) != 0 ||
		    power_supply_types[index].status != status)
			continue;
		power_supply_types[index].capacity = unit;
		_D("%s", power_supply_types[index].capacity);
		set_battery_power_supply(index);
	}
}

void battery_test_all(int *success, int *fail)
{
	int s = 0;
	int f = 0;

	(get_battery_time_to_full())    ? s++ : f++;
	(get_battery_time_to_empty())   ? s++ : f++;
	(get_battery_low_level(NULL))   ? s++ : f++;
	(set_battery_low_level(10))     ? s++ : f++;
	(set_battery_low_level(50))     ? s++ : f++;
	(get_battery_charger_status())  ? s++ : f++;
	(get_battery_charge_now())      ? s++ : f++;
	(get_battery_status_low())      ? s++ : f++;
	(get_battery_percent())         ? s++ : f++;
	(get_battery_percent_raw())     ? s++ : f++;
	(get_battery_is_full())         ? s++ : f++;
	(get_battery_health())          ? s++ : f++;
	(get_battery_info())            ? s++ : f++;

	for (int index = 0; index < ARRAY_SIZE(power_supply_types); index++)
		(set_battery_power_supply(index)) ? s++ : f++;

	if (NULL != success)    *success = s;
	if (NULL != fail)       *fail = f;
}

static void battery_init(void *data)
{
	int success = 0;
	int fail = 0;

	_I("Start test.");

	battery_test_all(&success, &fail);

	_I("Total=%d Success=%d Fail=%d", success+fail, success, fail);
}

static void battery_exit(void *data)
{
	_I("End test.");

}

static int battery_unit(int argc, char **argv)
{
	if (argc < 4) {
		int success = 0;
		int fail = 0;
		_I("Start test.");
		battery_test_all(&success, &fail);
		_I("Total=%d Success=%d Fail=%d", success+fail, success, fail);
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_TIMETOFULL)) {
		get_battery_time_to_full();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_TIMETOEMPTY)) {
		get_battery_time_to_empty();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_GETLOWBATLEVEL)) {
		get_battery_low_level(NULL);
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_SETLOWBATLEVEL)) {
		set_battery_low_level(atoi(argv[4]));
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_CHARGERSTATUS)) {
		get_battery_charger_status();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_CHARGENOW)) {
		get_battery_charge_now();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_STATUSLOW)) {
		get_battery_status_low();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_GETPERCENT)) {
		get_battery_percent();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_GETPERCENTRAW)) {
		get_battery_percent_raw();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_ISFULL)) {
		get_battery_is_full();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_HEALTH)) {
		get_battery_health();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_GETINFO)) {
		get_battery_info();
	} else if (0 == strcasecmp(argv[3], METHOD_BATTERY_POWERSUPPLY)) {
		for (int index = 0; index < ARRAY_SIZE(power_supply_types); index++)
			set_battery_power_supply(index);
	} else if (argc > 4 && argv[4] && !strncmp("enter", argv[4], 5)) {
			battery_scenario(argv[3], S_ENTER);
	} else if (argc > 4 && argv[4] && !strncmp("leave", argv[4], 5)) {
			battery_scenario(argv[3], S_LEAVE);
	} else if (argc >5 && argv[5] && !strncmp("enter", argv[5], 5)) {
			full_scenario(argv[3], argv[4], S_ENTER);
	} else if (argc >5 && argv[5] && !strncmp("leave", argv[5], 5)) {
			full_scenario(argv[3], argv[5], S_LEAVE);
	} else {
		_E("Unknown test case.");
	}

	return 0;
}

static const struct test_ops battery_test_ops = {
	.priority = TEST_PRIORITY_NORMAL,
	.name     = "battery",
	.init     = battery_init,
	.exit    = battery_exit,
	.unit    = battery_unit,
};

TEST_OPS_REGISTER(&battery_test_ops)
