/*
 * test
 *
 * Copyright (c) 2020 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "test.h"

static const struct device_change_type {
	char *name;
	char *status;
} device_change_types[] = {
	{"udev",		"start"},
	{"udev",		"stop"},
};

static int udev(int index)
{
	GVariant *msg;
	int ret = 0, val = 0;

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
			DEVICED_PATH_SYSNOTI,
			DEVICED_INTERFACE_SYSNOTI,
			"udev",
			g_variant_new("(sis)", "udev", 1, device_change_types[index].status));
	if (!msg) {
		_E("fail : %s %s %s %s",
			DEVICED_BUS_NAME, DEVICED_PATH_SYSNOTI, DEVICED_INTERFACE_SYSNOTI,
			"udev");
		return -EBADMSG;
	}

	if (!dh_get_param_from_var(msg, "(i)", &val))
		_E("fail : no message");
	else {
		if ((val == -ENOTSUP) || (val == -ENOSYS)) {
			_I("Not supported feature! : %d", val);
			ret = TRUE;
		} else if (val < 0) {
			_E("fail : returned fail (%d)", val);
		} else {
			_I("success : %d", val);
			ret = TRUE;
		}
	}

	_I("%s", device_change_types[index].status);
	if (val < 0)
		_R("[NG] ---- %s             : %s",
		__func__, device_change_types[index].status);
	else
		_R("[OK] ---- %s             : %s",
		__func__, device_change_types[index].status);

	g_variant_unref(msg);
	sleep(TEST_WAIT_TIME_INTERVAL);
	return ret;
}

static void unit(char *unit, char *status)
{
	int index;
	int unit_len;
	int status_len;

	if (!unit || !status)
		return;

	unit_len = strlen(unit);
	status_len = strlen(status);

	for (index = 0; index < ARRAY_SIZE(device_change_types); index++) {
		if (unit_len != strlen(device_change_types[index].name) ||
		    status_len != strlen(device_change_types[index].status))
			continue;
		if (strncmp(unit, device_change_types[index].name, unit_len) != 0 ||
		    strncmp(status, device_change_types[index].status, status_len) != 0)
			continue;
		udev(index);
	}
}

static void udev_init(void *data)
{
	int index;

	_I("start test");
	for (index = 0; index < ARRAY_SIZE(device_change_types); index++)
		udev(index);
}

static void udev_exit(void *data)
{
	_I("end test");
}

static int udev_unit(int argc, char **argv)
{
	if (argc != 4)
		return -EINVAL;

	unit(argv[2], argv[3]);
	return 0;
}

static const struct test_ops udev_test_ops = {
	.priority = TEST_PRIORITY_HIGH,
	.name     = "udev",
	.init     = udev_init,
	.exit    = udev_exit,
	.unit    = udev_unit,
};

TEST_OPS_REGISTER(&udev_test_ops)
