/*
 * deviced
 *
 * Copyright (c) 2012 - 2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <string.h>

#include "battery-ops.h"
#include "core/list.h"
#include "core/common.h"
#include "shared/log-macro.h"

static dd_list *batt_head;

void add_battery(const struct battery_ops *batt)
{
	DD_LIST_APPEND(batt_head, batt);
}

void remove_battery(const struct battery_ops *batt)
{
	DD_LIST_REMOVE(batt_head, batt);
}

const struct battery_ops *find_battery_feature(const char *name)
{
	dd_list *elem;
	const struct battery_ops *batt;

	DD_LIST_FOREACH(batt_head, elem, batt) {
		if (!strcmp(batt->name, name))
			return batt;
	}
	return NULL;
}

void battery_ops_init(void *data)
{
	dd_list *elem;
	const struct battery_ops *batt;

	DD_LIST_FOREACH(batt_head, elem, batt) {
		_D("[%s] initialize.", batt->name);
		if (batt->init)
			batt->init(data);
	}
}

void battery_ops_exit(void *data)
{
	dd_list *elem;
	const struct battery_ops *batt;

	DD_LIST_FOREACH(batt_head, elem, batt) {
		_D("[%s] deinitialize.", batt->name);
		if (batt->exit)
			batt->exit(data);
	}
}
